﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Interactivity;
using System.Windows;
using System.Windows.Controls;
using Infragistics;
using Infragistics.Windows.DataPresenter;
using Infragistics.Windows;
using System.Windows.Controls.Primitives;
using System.Windows.Media;

namespace WpfApplication1.Behaviors
{
    public class GridTooSmallBehavior : Behavior<XamDataGrid>
    {
        public XamDataGrid Grid { get { return AssociatedObject; } }

        #region Parent Property

        public Border Parent
        {
            get { return (Border)GetValue(ParentProperty); }
            set { SetValue(ParentProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Parent.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ParentProperty =
            DependencyProperty.Register("Parent", typeof(Border), typeof(GridTooSmallBehavior), new UIPropertyMetadata(null));

        #endregion

        protected override void OnAttached()
        {
            base.OnAttached();

            Grid.SizeChanged += new SizeChangedEventHandler(Grid_SizeChanged);
            //Grid.Field += new EventHandler<ColumnFixedStateEventArgs>(Grid_ColumnFixedStateChanged);
            Grid.FieldPositionChanged += Grid_FieldPositionChanged;

            Style labelPresenterStyle = new Style(typeof(LabelPresenter));
            labelPresenterStyle.Setters.Add(new EventSetter(LabelPresenter.SizeChangedEvent, new SizeChangedEventHandler(OnFieldSizeChanged)));
            Grid.Resources.Add(typeof(LabelPresenter), labelPresenterStyle);
        }

        void Grid_FieldPositionChanged(object sender, Infragistics.Windows.DataPresenter.Events.FieldPositionChangedEventArgs e)
        {
            if (e.ChangeReason == FieldPositionChangeReason.Fixed)
            {
                AdjustMargin();
            }
        }

        void OnFieldSizeChanged(object sender, SizeChangedEventArgs e)
        {
            AdjustMargin();
        }

        ScrollBar FindScrollbar(DependencyObject parent, Orientation orientation)
        {
            if (parent is ScrollBar && (parent as ScrollBar).Orientation == orientation)
                return parent as ScrollBar;

            for (int i = 0; i < VisualTreeHelper.GetChildrenCount(parent); i++)
            {
                DependencyObject child = VisualTreeHelper.GetChild(parent, i);
                if (child is ScrollBar && (child as ScrollBar).Orientation == orientation)
                    return child as ScrollBar;

                ScrollBar result = FindScrollbar(child, orientation);
                if (result != null)
                    return result;
            }

            return null;
        }

        void AdjustMargin()
        {
            ScrollViewer sv = (ScrollViewer)Utilities.GetDescendantFromType(Grid, typeof(ScrollViewer), false);
            ScrollBar horizontalScrollbar = FindScrollbar(sv, Orientation.Horizontal);

            horizontalScrollbar.Margin = new Thickness(CalculateTotalFixedColumnWidths(), 0, 0, 0);
        }

        void Grid_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            double fixedColumnsWidth = CalculateTotalFixedColumnWidths();

            // the fixed columns are wider than the actual grid so put it in a ScrollViewer
            if (fixedColumnsWidth >= Grid.ActualWidth && Parent != null)
            {
                // detach the grid from the parent.
                Parent.Child = null;

                // create the scrollviewer
                ScrollViewer viewer = new ScrollViewer();
                viewer.HorizontalScrollBarVisibility = ScrollBarVisibility.Visible;
                viewer.VerticalScrollBarVisibility = ScrollBarVisibility.Hidden;
                viewer.SizeChanged += new SizeChangedEventHandler(viewer_SizeChanged);
                viewer.Content = Grid;

                // give the grid a width greater than the fixed columns
                Grid.Width = fixedColumnsWidth + 150;

                // adjust the grid height so that we won't need the vertical scrollbar
                IProvideScrollInfo scrollInfo = Grid as IProvideScrollInfo;
                ScrollViewer sv = (ScrollViewer)Utilities.GetDescendantFromType(Grid, typeof(ScrollViewer), false);
                ScrollBar horizontalScrollbar = FindScrollbar(sv, Orientation.Horizontal);

                Grid.Height = Parent.ActualHeight - horizontalScrollbar.ActualHeight + 2;

                // in order to see the change the scrollviewer needs to be assigned the grid's old parent.
                Parent.Child = viewer;
            }
        }

        void viewer_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            // when the scrollviewer height changes, adjust the grid's height.
            IProvideScrollInfo scrollInfo = Grid as IProvideScrollInfo;
            ScrollViewer sv = (ScrollViewer)Utilities.GetDescendantFromType(Grid, typeof(ScrollViewer), false);
            ScrollBar horizontalScrollbar = FindScrollbar(sv, Orientation.Horizontal);

            Grid.Height = Parent.ActualHeight - horizontalScrollbar.ActualHeight + 2;

            // when the scrollviewer's width is greater than the grid's width, put the grid back the way it was.
            if (e.NewSize.Width > Grid.Width)
            {
                // cleanup the scrollviewer
                (sender as ScrollViewer).Content = null;
                (sender as ScrollViewer).SizeChanged -= viewer_SizeChanged;
                Parent.Child = null;

                // set the grid's width and height to NaN so it uses it's parent size
                Grid.Width = double.NaN;
                Grid.Height = double.NaN;
                Parent.Child = Grid;
            }
        }

        double CalculateTotalFixedColumnWidths()
        {
            double fixedColumnsWidth = 0.0;
            foreach (Field f in Grid.FieldLayouts[0].Fields)
            {
                if (f.IsFixed)
                    fixedColumnsWidth += f.CellWidthResolved;
            }

            // I add 20 to account for the record selector
            return fixedColumnsWidth + 20;
        }
    }
}
